"""
Exports the definition of polynomial systems, in double, double double,
and quad double precision.  Also polynomials with negative exponents,
the so-called Laurent polynomials, are supported.
"""
from ctypes import c_int32, c_double, pointer, create_string_buffer
from phcpy.version import get_phcfun, int4a2nbr, str2int4a, int4a2str
from phcpy.dimension import set_double_dimension, get_double_dimension
from phcpy.dimension import set_double_double_dimension
from phcpy.dimension import get_double_double_dimension
from phcpy.dimension import set_quad_double_dimension
from phcpy.dimension import get_quad_double_dimension
from phcpy.dimension import set_double_laurent_dimension
from phcpy.dimension import get_double_laurent_dimension
from phcpy.dimension import set_double_double_laurent_dimension
from phcpy.dimension import get_double_double_laurent_dimension
from phcpy.dimension import set_quad_double_laurent_dimension
from phcpy.dimension import get_quad_double_laurent_dimension

def set_double_polynomial(idx, nvr, pol, vrblvl=0):
    """
    Sets the polynomial by the string pol, in double precision,
    with a number of variables no more than nvr, at index idx.
    The index starts at one, not at zero.
    This function does not set the dimension,
    which must be set to a value at least idx.
    """
    if vrblvl > 0:
        print('in set_double_polynomial, pol :', pol, end='')
        print(', idx :', idx, ', nvr :', nvr)
    phc = get_phcfun(vrblvl-1)
    apars = int4a2nbr([len(pol), nvr, idx], vrblvl=vrblvl-1)
    bpol = str2int4a(pol, vrblvl=vrblvl-1)
    ccc = pointer(c_double(0.0))
    vlvl = c_int32(vrblvl)
    if vrblvl > 0:
        print('-> set_double_polynomial calls phc', end='')
    retval = phc(76, apars, bpol, ccc, vlvl)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_double_double_polynomial(idx, nvr, pol, vrblvl=0):
    """
    Sets the polynomial by the string pol, in double double precision,
    with a number of variables no more than nvr, at index idx.
    The index starts at one, not at zero.
    This function does not set the dimension,
    which must be set to a value at least idx.
    """
    if vrblvl > 0:
        print('in set_double_double_polynomial, pol :', pol, end='')
        print(', idx :', idx, ', nvr :', nvr)
    phc = get_phcfun(vrblvl-1)
    apars = int4a2nbr([len(pol), nvr, idx], vrblvl=vrblvl-1)
    bpol = str2int4a(pol, vrblvl=vrblvl-1)
    ccc = pointer(c_double(0.0))
    vlvl = c_int32(vrblvl)
    if vrblvl > 0:
        print('-> set_double_double_polynomial calls phc', end='')
    retval = phc(338, apars, bpol, ccc, vlvl)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_quad_double_polynomial(idx, nvr, pol, vrblvl=0):
    """
    Sets the polynomial by the string pol, in quad double precision,
    with a number of variables no more than nvr, at index idx.
    The index starts at one, not at zero.
    This function does not set the dimension,
    which must be set to a value at least idx.
    """
    if vrblvl > 0:
        print('in set_quad_double_polynomial, pol :', pol, end='')
        print(', idx :', idx, ', nvr :', nvr)
    phc = get_phcfun(vrblvl-1)
    apars = int4a2nbr([len(pol), nvr, idx], vrblvl=vrblvl-1)
    bpol = str2int4a(pol, vrblvl=vrblvl-1)
    ccc = pointer(c_double(0.0))
    vlvl = c_int32(vrblvl)
    if vrblvl > 0:
        print('-> set_quad_double_polynomial calls phc', end='')
    retval = phc(388, apars, bpol, ccc, vlvl)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_double_laurent_polynomial(idx, nvr, pol, vrblvl=0):
    """
    Sets the polynomial by the string pol, in double precision,
    with a number of variables no more than nvr, at index idx.
    The index starts at one, not at zero.
    This function does not set the dimension,
    which must be set to a value at least idx.
    """
    if vrblvl > 0:
        print('in set_double_laurent_polynomial, pol :', pol, end='')
        print(', idx :', idx, ', nvr :', nvr)
    phc = get_phcfun(vrblvl-1)
    apars = int4a2nbr([len(pol), nvr, idx], vrblvl=vrblvl-1)
    bpol = str2int4a(pol, vrblvl=vrblvl-1)
    ccc = pointer(c_double(0.0))
    vlvl = c_int32(vrblvl)
    if vrblvl > 0:
        print('-> set_double_laurent_polynomial calls phc', end='')
    retval = phc(74, apars, bpol, ccc, vlvl)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_double_double_laurent_polynomial(idx, nvr, pol, vrblvl=0):
    """
    Sets the polynomial by the string pol, in double double precision,
    with a number of variables no more than nvr, at index idx.
    The index starts at one, not at zero.
    This function does not set the dimension,
    which must be set to a value at least idx.
    """
    if vrblvl > 0:
        print('in set_double_double_laurent_polynomial, pol :', pol, end='')
        print(', idx :', idx, ', nvr :', nvr)
    phc = get_phcfun(vrblvl-1)
    apars = int4a2nbr([len(pol), nvr, idx], vrblvl=vrblvl-1)
    bpol = str2int4a(pol, vrblvl=vrblvl-1)
    ccc = pointer(c_double(0.0))
    vlvl = c_int32(vrblvl)
    if vrblvl > 0:
        print('-> set_double_double_laurent_polynomial calls phc', end='')
    retval = phc(558, apars, bpol, ccc, vlvl)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_quad_double_laurent_polynomial(idx, nvr, pol, vrblvl=0):
    """
    Sets the polynomial by the string pol, in quad double precision,
    with a number of variables no more than nvr, at index idx.
    The index starts at one, not at zero.
    This function does not set the dimension,
    which must be set to a value at least idx.
    """
    if vrblvl > 0:
        print('in set_quad_double_laurent_polynomial, pol :', pol, end='')
        print(', idx :', idx, ', nvr :', nvr)
    phc = get_phcfun(vrblvl-1)
    apars = int4a2nbr([len(pol), nvr, idx], vrblvl=vrblvl-1)
    bpol = str2int4a(pol, vrblvl=vrblvl-1)
    ccc = pointer(c_double(0.0))
    vlvl = c_int32(vrblvl)
    if vrblvl > 0:
        print('-> set_quad_double_laurent_polynomial calls phc', end='')
    retval = phc(568, apars, bpol, ccc, vlvl)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_double_system(nvr, pols, vrblvl=0):
    """
    Sets the system defines by the strings in pols, in double precision,
    with a number of variables no more than nvr.
    The dimension of the system is set to len(pols).
    Returns the sum of the return values of set_double_polynomial.
    """
    if vrblvl > 0:
        print('in set_double_system, nvr :', nvr)
    dim = len(pols)
    set_double_dimension(dim, vrblvl)
    result = 0
    for k in range(dim):
        if vrblvl > 0:
            print(f"-> set_double_system, pols[{k}] = {pols[k]}")
        retval = set_double_polynomial(k+1, nvr, pols[k], vrblvl)
        if vrblvl > 0:
            print('-> set_double_polynomial returns', retval)
        result = result + retval
    return result

def set_double_double_system(nvr, pols, vrblvl=0):
    """
    Sets the system defines by the strings in pols, in double double precision,
    with a number of variables no more than nvr.
    The dimension of the system is set to len(pols).
    Returns the sum of the return values of set_double_double_polynomial.
    """
    if vrblvl > 0:
        print('in set_double_double_system, nvr :', nvr)
    dim = len(pols)
    set_double_double_dimension(dim, vrblvl)
    result = 0
    for k in range(dim):
        if vrblvl > 0:
            print(f"-> set_double_double_system, pols[{k}] = {pols[k]}")
        retval = set_double_double_polynomial(k+1, nvr, pols[k], vrblvl)
        if vrblvl > 0:
            print('-> set_double_double_polynomial returns', retval)
        result = result + retval
    return result

def set_quad_double_system(nvr, pols, vrblvl=0):
    """
    Sets the system defines by the strings in pols, in quad double precision,
    with a number of variables no more than nvr.
    The dimension of the system is set to len(pols).
    Returns the sum of the return values of set_quad_double_polynomial.
    """
    if vrblvl > 0:
        print('in set_quad_double_system, nvr :', nvr)
    dim = len(pols)
    set_quad_double_dimension(dim, vrblvl)
    result = 0
    for k in range(dim):
        if vrblvl > 0:
            print(f"-> set_quad_double_system, pols[{k}] = {pols[k]}")
        retval = set_quad_double_polynomial(k+1, nvr, pols[k], vrblvl)
        if vrblvl > 0:
            print('-> set_quad_double_polynomial returns', retval)
        result = result + retval
    return result

def set_double_laurent_system(nvr, pols, vrblvl=0):
    """
    Sets the laurent system defines by the strings in pols,
    in double precision,
    with a number of variables no more than nvr.
    The dimension of the system is set to len(pols).
    Returns the sum of the return values of set_double_laurent_polynomial.
    """
    if vrblvl > 0:
        print('in set_double_laurent_system, nvr :', nvr)
    dim = len(pols)
    set_double_laurent_dimension(dim, vrblvl)
    result = 0
    for k in range(dim):
        if vrblvl > 0:
            print(f"-> set_double_laurent_system, pols[{k}] = {pols[k]}")
        retval = set_double_laurent_polynomial(k+1, nvr, pols[k], vrblvl)
        if vrblvl > 0:
            print('-> set_double_laurent_polynomial returns', retval)
        result = result + retval
    return result

def set_double_double_laurent_system(nvr, pols, vrblvl=0):
    """
    Sets the laurent system defines by the strings in pols,
    in double double precision,
    with a number of variables no more than nvr.
    The dimension of the system is set to len(pols).
    Returns the sum of the return values 
    of set_double_double_laurent_polynomial.
    """
    if vrblvl > 0:
        print('in set_double_double_laurent_system, nvr :', nvr)
    dim = len(pols)
    set_double_double_laurent_dimension(dim, vrblvl)
    result = 0
    for k in range(dim):
        if vrblvl > 0:
            print(f"-> set_double_double_laurent_system, pols[{k}] = {pols[k]}")
        retval = set_double_double_laurent_polynomial(k+1, nvr, pols[k], vrblvl)
        if vrblvl > 0:
            print('-> set_double_double_laurent_polynomial returns', retval)
        result = result + retval
    return result

def set_quad_double_laurent_system(nvr, pols, vrblvl=0):
    """
    Sets the laurent system defines by the strings in pols,
    in quad double precision,
    with a number of variables no more than nvr.
    The dimension of the system is set to len(pols).
    Returns the sum of the return values of set_quad_double_polynomial.
    """
    if vrblvl > 0:
        print('in set_quad_double_laurent_system, nvr :', nvr)
    dim = len(pols)
    set_quad_double_laurent_dimension(dim, vrblvl)
    result = 0
    for k in range(dim):
        if vrblvl > 0:
            print(f"-> set_quad_double_laurent_system, pols[{k}] = {pols[k]}")
        retval = set_quad_double_laurent_polynomial(k+1, nvr, pols[k], vrblvl)
        if vrblvl > 0:
            print('-> set_quad_double_laurent_polynomial returns', retval)
        result = result + retval
    return result

def get_double_polynomial(idx, vrblvl=0):
    """
    Returns the string representation of the polynomial
    in double precision, at index idx.
    """
    if vrblvl > 0:
        print('in get_double_polynomial idx :', idx)
    phc = get_phcfun(vrblvl-1)
    adx = pointer(c_int32(idx))
    bsz = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_double_polynomial calls phc', end='')
    retval = phc(600, adx, bsz, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('-> size of the polynomial :', bsz[0])
    szd = bsz[0]
    poldata = create_string_buffer(b"", 4*szd)
    if vrblvl > 0:
        print('-> get_double_polynomial calls phc', end='')
    retval = phc(67, adx, poldata, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    strpol = int4a2str(poldata, vrblvl=vrblvl-1)
    pols = strpol.split(';')
    result = pols[0] + ';'
    return result

def get_double_double_polynomial(idx, vrblvl=0):
    """
    Returns the string representation of the polynomial
    in double double precision, at index idx.
    """
    if vrblvl > 0:
        print('in get_double_double_polynomial idx :', idx)
    phc = get_phcfun(vrblvl-1)
    adx = pointer(c_int32(idx))
    bsz = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_double_double_polynomial calls phc', end='')
    retval = phc(601, adx, bsz, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('-> size of the polynomial :', bsz[0])
    szd = bsz[0]
    poldata = create_string_buffer(b"", 4*szd)
    if vrblvl > 0:
        print('-> get_double_double_polynomial calls phc', end='')
    retval = phc(106, adx, poldata, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    strpol = int4a2str(poldata, vrblvl=vrblvl-1)
    pols = strpol.split(';')
    result = pols[0] + ';'
    return result

def get_quad_double_polynomial(idx, vrblvl=0):
    """
    Returns the string representation of the polynomial
    in quad double precision, at index idx.
    """
    if vrblvl > 0:
        print('in get_quad_double_polynomial idx :', idx)
    phc = get_phcfun(vrblvl-1)
    adx = pointer(c_int32(idx))
    bsz = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_quad_double_polynomial calls phc', end='')
    retval = phc(602, adx, bsz, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('-> size of the polynomial :', bsz[0])
    szd = bsz[0]
    poldata = create_string_buffer(b"", 4*szd)
    if vrblvl > 0:
        print('-> get_quad_double_polynomial calls phc', end='')
    retval = phc(107, adx, poldata, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    strpol = int4a2str(poldata, vrblvl=vrblvl-1)
    pols = strpol.split(';')
    result = pols[0] + ';'
    return result

def get_double_laurent_polynomial(idx, vrblvl=0):
    """
    Returns the string representation of the Laurent polynomial
    in double precision, at index idx.
    """
    if vrblvl > 0:
        print('in get_double_laurent_polynomial idx :', idx)
    phc = get_phcfun(vrblvl-1)
    adx = pointer(c_int32(idx))
    bsz = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_double_laurent_polynomial calls phc', end='')
    retval = phc(604, adx, bsz, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('-> size of the polynomial :', bsz[0])
    szd = bsz[0]
    poldata = create_string_buffer(b"", 4*szd)
    if vrblvl > 0:
        print('-> get_double_laurent_polynomial calls phc', end='')
    retval = phc(128, adx, poldata, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    strpol = int4a2str(poldata, vrblvl=vrblvl-1)
    pols = strpol.split(';')
    result = pols[0] + ';'
    return result

def get_double_double_laurent_polynomial(idx, vrblvl=0):
    """
    Returns the string representation of the Laurent polynomial
    in double double precision, at index idx.
    """
    if vrblvl > 0:
        print('in get_double_double_laurent_polynomial idx :', idx)
    phc = get_phcfun(vrblvl-1)
    adx = pointer(c_int32(idx))
    bsz = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_double_double_laurent_polynomial calls phc', end='')
    retval = phc(605, adx, bsz, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('-> size of the polynomial :', bsz[0])
    szd = bsz[0]
    poldata = create_string_buffer(b"", 4*szd)
    if vrblvl > 0:
        print('-> get_double_double_laurent_polynomial calls phc', end='')
    retval = phc(559, adx, poldata, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    strpol = int4a2str(poldata, vrblvl=vrblvl-1)
    pols = strpol.split(';')
    result = pols[0] + ';'
    return result

def get_quad_double_laurent_polynomial(idx, vrblvl=0):
    """
    Returns the string representation of the Laurent polynomial
    in quad double precision, at index idx.
    """
    if vrblvl > 0:
        print('in get_quad_double_laurent_polynomial idx :', idx)
    phc = get_phcfun(vrblvl-1)
    adx = pointer(c_int32(idx))
    bsz = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_quad_double_laurent_polynomial calls phc', end='')
    retval = phc(606, adx, bsz, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('-> size of the polynomial :', bsz[0])
    szd = bsz[0]
    poldata = create_string_buffer(b"", 4*szd)
    if vrblvl > 0:
        print('-> get_quad_double_laurent_polynomial calls phc', end='')
    retval = phc(569, adx, poldata, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    strpol = int4a2str(poldata, vrblvl=vrblvl-1)
    pols = strpol.split(';')
    result = pols[0] + ';'
    return result

def get_double_system(vrblvl=0):
    """
    Returns the string representation of the polynomials
    in double precision.
    """
    dim = get_double_dimension(vrblvl)
    if vrblvl > 0:
        print('in get_double_system, dim :', dim)
    result = []
    for k in range(1, dim+1):
        pol = get_double_polynomial(k, vrblvl)
        if vrblvl > 0:
            print('get_double_system retrieved pol :', pol)
        result.append(pol)
    return result

def get_double_double_system(vrblvl=0):
    """
    Returns the string representation of the polynomials
    in double double precision.
    """
    dim = get_double_double_dimension(vrblvl)
    if vrblvl > 0:
        print('in get_double_double_system, dim :', dim)
    result = []
    for k in range(1, dim+1):
        pol = get_double_double_polynomial(k, vrblvl)
        if vrblvl > 0:
            print('-> get_double_double_system retrieved pol :', pol)
        result.append(pol)
    return result

def get_quad_double_system(vrblvl=0):
    """
    Returns the string representation of the polynomials
    in quad double precision.
    """
    dim = get_quad_double_dimension(vrblvl)
    if vrblvl > 0:
        print('in get_quad_double_system, dim :', dim)
    result = []
    for k in range(1, dim+1):
        pol = get_quad_double_polynomial(k, vrblvl)
        if vrblvl > 0:
            print('get_quad_double_system retrieved pol :', pol)
        result.append(pol)
    return result

def get_double_laurent_system(vrblvl=0):
    """
    Returns the string representation of the Laurent polynomials
    in double precision.
    """
    dim = get_double_laurent_dimension(vrblvl)
    if vrblvl > 0:
        print('in get_double_laurent_system, dim :', dim)
    result = []
    for k in range(1, dim+1):
        pol = get_double_laurent_polynomial(k, vrblvl)
        if vrblvl > 0:
            print('get_double_laurent_system retrieved pol :', pol)
        result.append(pol)
    return result

def get_double_double_laurent_system(vrblvl=0):
    """
    Returns the string representation of the Laurent polynomials
    in double double precision.
    """
    dim = get_double_double_laurent_dimension(vrblvl)
    if vrblvl > 0:
        print('in get_double_double_laurent_system, dim :', dim)
    result = []
    for k in range(1, dim+1):
        pol = get_double_double_laurent_polynomial(k, vrblvl)
        if vrblvl > 0:
            print('get_double_double_laurent_system retrieved pol :', pol)
        result.append(pol)
    return result

def get_quad_double_laurent_system(vrblvl=0):
    """
    Returns the string representation of the Laurent polynomials
    in quad double precision.
    """
    dim = get_quad_double_laurent_dimension(vrblvl)
    if vrblvl > 0:
        print('in get_quad_double_laurent_system, dim :', dim)
    result = []
    for k in range(1, dim+1):
        pol = get_quad_double_laurent_polynomial(k, vrblvl)
        if vrblvl > 0:
            print('get_quad_double_laurent_system retrieved pol :', pol)
        result.append(pol)
    return result

def get_double_number_terms(equ, vrblvl=0):
    """
    Returns the number of terms in the equation with index equ,
    set as a polynomial system in double precision.
    """
    if vrblvl > 0:
        print('in get_double_number_terms, equ :', equ)
    phc = get_phcfun(vrblvl-1)
    idx = (c_int32 * 2)()
    idx[0] = equ
    idx[1] = equ
    aidx = pointer(idx)
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_double_number_of_terms calls phc', end='')
    retval = phc(24, aidx, bbb, ccc, vrb)
    result = aidx[0][0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('number of terms :', result)
    return result

def get_double_number_laurent_terms(equ, vrblvl=0):
    """
    Returns the number of terms in the equation with index equ,
    set as a Laurent system in double precision.
    """
    if vrblvl > 0:
        print('in get_double_number_laurent_terms, equ :', equ)
    phc = get_phcfun(vrblvl-1)
    idx = (c_int32 * 2)()
    idx[0] = equ
    idx[1] = equ
    aidx = pointer(idx)
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_double_number_of_laurent_terms calls phc', end='')
    retval = phc(124, aidx, bbb, ccc, vrb)
    result = aidx[0][0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('number of terms :', result)
    return result

def get_double_double_number_terms(equ, vrblvl=0):
    """
    Returns the number of terms in the equation with index equ,
    set as a polynomial system in double double precision.
    """
    if vrblvl > 0:
        print('in get_double_double_number_terms, equ :', equ)
    phc = get_phcfun(vrblvl-1)
    idx = (c_int32 * 2)()
    idx[0] = equ
    idx[1] = equ
    aidx = pointer(idx)
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_double_double_number_of_terms calls phc', end='')
    retval = phc(334, aidx, bbb, ccc, vrb)
    result = aidx[0][0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('number of terms :', result)
    return result

def get_double_double_number_laurent_terms(equ, vrblvl=0):
    """
    Returns the number of terms in the equation with index equ,
    set as a Laurent system in double double precision.
    """
    if vrblvl > 0:
        print('in get_double_double_number_laurent_terms, equ :', equ)
    phc = get_phcfun(vrblvl-1)
    idx = (c_int32 * 2)()
    idx[0] = equ
    idx[1] = equ
    aidx = pointer(idx)
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_double_double_number_of_laurent_terms calls phc', end='')
    retval = phc(554, aidx, bbb, ccc, vrb)
    result = aidx[0][0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('number of terms :', result)
    return result

def get_quad_double_number_terms(equ, vrblvl=0):
    """
    Returns the number of terms in the equation with index equ,
    set as a polynomial system in quad double precision.
    """
    if vrblvl > 0:
        print('in get_quad_double_number_terms, equ :', equ)
    phc = get_phcfun(vrblvl-1)
    idx = (c_int32 * 2)()
    idx[0] = equ
    idx[1] = equ
    aidx = pointer(idx)
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_quad_double_number_of_terms calls phc', end='')
    retval = phc(384, aidx, bbb, ccc, vrb)
    result = aidx[0][0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('number of terms :', result)
    return result

def get_quad_double_number_laurent_terms(equ, vrblvl=0):
    """
    Returns the number of terms in the equation with index equ,
    set as a Laurent system in quad double precision.
    """
    if vrblvl > 0:
        print('in get_quad_double_number_laurent_terms, equ :', equ)
    phc = get_phcfun(vrblvl-1)
    idx = (c_int32 * 2)()
    idx[0] = equ
    idx[1] = equ
    aidx = pointer(idx)
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_quad_double_number_of_laurent_terms calls phc', end='')
    retval = phc(564, aidx, bbb, ccc, vrb)
    result = aidx[0][0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('number of terms :', result)
    return result

def string_of_symbols(maxlen=100, vrblvl=0):
    """
    Returns the list of all symbols (as strings),
    defined when storing polynomials.
    The maxlen on entry equals the maximum number of characters
    in the symbol string, that is: the sequence of all string
    representations of the symbols, separated by one space.
    """
    if vrblvl > 0:
        print('in string_of_symbols, maxlen :', maxlen)
    phc = get_phcfun(vrblvl-1)
    slen = pointer(c_int32(0))
    ssym = create_string_buffer(b"", maxlen*4)
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> string_of_symbols calls phc', end='')
    retval = phc(295, slen, ssym, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('number of characters :', slen[0])
    variables = int4a2str(ssym, vrblvl=vrblvl-1)
    result = variables[0:slen[0]].split(' ')
    return result

def number_of_symbols(pols, vrblvl=0):
    """
    Returns the number of symbols that appear as variables
    in the polynomials, givein in the list of strings pols.
    Useful to determine whether a system is square or not.
    """
    if vrblvl > 0:
        print('in number_of_symbols, pols :')
        for pol in pols:
            print(pol)
    inpols = ''.join(pols)
    phc = get_phcfun(vrblvl-1)
    slen = pointer(c_int32(len(inpols)))
    bpol = str2int4a(inpols)
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> number_of_symbols calls phc', end='')
    retval = phc(439, slen, bpol, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('-> number_of_symbols, result :', slen[0])
    return slen[0]

def check_semicolons(pols, vrblvl=0):
    """
    Counts the number of semicolons and writes a warning
    if the number of semicolons in pols does not match len(pols).
    This warning may prevent unwanted concatenation, e.g., as in
    pols = ['x^2 + 4*y^2 - 4;' '2*y^2 - x;']
    where the omitted comma results in one polynomial in two variables.
    """
    if vrblvl > 0:
        print('in check_semicolons, pols :')
        for pol in pols:
            print(pol)
    checksum = sum(pol.count(';') for pol in pols)
    if checksum != len(pols):
        print('#semicolons :', checksum, '!=', len(pols), 'polynomials.')
    return int(checksum != len(pols))

def is_square(pols, vrblvl=0):
    r"""
    Given in the list *pols* are string representations 
    of Laurent polynomials.
    A system is square if it has as many unknowns as equations.
    Returns True if the system is square, False otherwise.
    """
    if vrblvl > 0:
        print('in is_square, pols :')
        for pol in pols:
            print(pol)
    nbrvar = number_of_symbols(pols)
    nbreqs = len(pols)
    return nbrvar == nbreqs

def degree_of_double_polynomial(idx, vrblvl=0):
    """
    Returns the degree of the polynomial with double precision
    coefficients stored at position idx.
    """
    if vrblvl > 0:
        print('in degree_of_double_polynomial, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(idx))
    bdeg = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> degree_of_double_system calls phc', end='')
    retval = phc(119, aidx, bdeg, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('-> degree of polynomial :', bdeg[0])
    return bdeg[0]

def clear_double_system(vrblvl=0):
    """
    Clears the system set in double precision.
    """
    if vrblvl > 0:
        print('in clear_double_system ...')
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_system calls phc', end='')
    retval = phc(27, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_double_double_system(vrblvl=0):
    """
    Clears the system set in double double precision.
    """
    if vrblvl > 0:
        print('in clear_double_double_system ...')
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_double_system calls phc', end='')
    retval = phc(337, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_quad_double_system(vrblvl=0):
    """
    Clears the system set in quad double precision.
    """
    if vrblvl > 0:
        print('in clear_quad_double_system ...')
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_quad_double_system calls phc', end='')
    retval = phc(387, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_double_laurent_system(vrblvl=0):
    """
    Clears the Laurent system set in double precision.
    """
    if vrblvl > 0:
        print('in clear_double_laurent_system ...')
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_laurent_system calls phc', end='')
    retval = phc(127, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_double_double_laurent_system(vrblvl=0):
    """
    Clears the Laurent system set in double double precision.
    """
    if vrblvl > 0:
        print('in clear_double_double_laurent_system ...')
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_double_laurent_system calls phc', end='')
    retval = phc(557, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_quad_double_laurent_system(vrblvl=0):
    """
    Clears the Laurent system set in quad double precision.
    """
    if vrblvl > 0:
        print('in clear_quad_double_laurent_system ...')
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_quad_double_laurent_system calls phc', end='')
    retval = phc(567, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_symbol_table(vrblvl=0):
    """
    Clears the table of symbols used to represent polynomials.
    """
    if vrblvl > 0:
        print('in clear_symbol_table ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_symbol_table calls phc', end='')
    retval = phc(29, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def initialize_double_syspool(dim, vrblvl=0):
    """
    Initialize the systems pool in double precision with dim.
    """
    if vrblvl > 0:
        print('in initialize_double_syspool, dim :', dim)
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(dim))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> initialize_double_syspool calls phc', end='')
    retval = phc(300, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def initialize_double_double_syspool(dim, vrblvl=0):
    """
    Initialize the systems pool in double double precision with dim.
    """
    if vrblvl > 0:
        print('in initialize_double_double_syspool, dim :', dim)
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(dim))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> initialize_double_double_syspool calls phc', end='')
    retval = phc(318, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def initialize_quad_double_syspool(dim, vrblvl=0):
    """
    Initialize the systems pool in quad double precision with dim.
    """
    if vrblvl > 0:
        print('in initialize_quad_double_syspool, dim :', dim)
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(dim))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> initialize_quad_double_syspool calls phc', end='')
    retval = phc(319, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def size_double_syspool(vrblvl=0):
    """
    Returns the size of the systems pool in double precision.
    """
    if vrblvl > 0:
        print('in size_double_syspool ...')
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> size_double_syspool calls phc', end='')
    retval = phc(301, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('number of systems in the pool :', adim[0])
    return adim[0]

def size_double_double_syspool(vrblvl=0):
    """
    Returns the size of the systems pool in double double precision.
    """
    if vrblvl > 0:
        print('in size_double_double_syspool ...')
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> size_double_double_syspool calls phc', end='')
    retval = phc(316, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('number of systems in the pool :', adim[0])
    return adim[0]

def size_quad_double_syspool(vrblvl=0):
    """
    Returns the size of the systems pool in quad double precision.
    """
    if vrblvl > 0:
        print('in size_quad_double_syspool ...')
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> size_quad_double_syspool calls phc', end='')
    retval = phc(317, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('number of systems in the pool :', adim[0])
    return adim[0]

def copy_to_double_syspool(idx, vrblvl=0):
    """
    Copies the system set in double precision to position idx
    in the systems pool.
    """
    if vrblvl > 0:
        print('in copy_to_double_syspool, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(idx))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_to_double_syspool calls phc', end='')
    retval = phc(304, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_to_double_double_syspool(idx, vrblvl=0):
    """
    Copies the system set in double double precision to position idx
    in the systems pool.
    """
    if vrblvl > 0:
        print('in copy_to_double_double_syspool, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(idx))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_to_double_double_syspool calls phc', end='')
    retval = phc(608, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_to_quad_double_syspool(idx, vrblvl=0):
    """
    Copies the system set in quad double precision to position idx
    in the systems pool.
    """
    if vrblvl > 0:
        print('in copy_to_quad_double_syspool, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(idx))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_to_quad_double_syspool calls phc', end='')
    retval = phc(609, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_from_double_syspool(idx, vrblvl=0):
    """
    Copies the system in double precision at position idx
    in the systems pool to the defined system in double precision.
    """
    if vrblvl > 0:
        print('in copy_from_double_syspool, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(idx))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_from_double_syspool calls phc', end='')
    retval = phc(313, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_from_double_double_syspool(idx, vrblvl=0):
    """
    Copies the system in double double precision at position idx
    in the systems pool to the defined system in double double precision.
    """
    if vrblvl > 0:
        print('in copy_from_double_double_syspool, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(idx))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_from_double_double_syspool calls phc', end='')
    retval = phc(314, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def copy_from_quad_double_syspool(idx, vrblvl=0):
    """
    Copies the system in quad double precision at position idx
    in the systems pool to the defined system in quad double precision.
    """
    if vrblvl > 0:
        print('in copy_from_quad_double_syspool, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(idx))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> copy_from_quad_double_syspool calls phc', end='')
    retval = phc(315, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_double_syspool(vrblvl=0):
    """
    Clears the systems pool in double precision.
    """
    if vrblvl > 0:
        print('in clear_double_syspool ...')
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_syspool calls phc', end='')
    retval = phc(697, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_double_double_syspool(vrblvl=0):
    """
    Clears the systems pool in double double precision.
    """
    if vrblvl > 0:
        print('in clear_double_double_syspool ...')
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_double_syspool calls phc', end='')
    retval = phc(698, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_quad_double_syspool(vrblvl=0):
    """
    Clears the systems pool in quad double precision.
    """
    if vrblvl > 0:
        print('in clear_quad_double_syspool ...')
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_quad_double_syspool calls phc', end='')
    retval = phc(699, adim, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def test_double_polynomial(vrblvl=0):
    """
    Tests the setting and getting of a polynomial, in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_polynomial ...')
    set_double_dimension(2, vrblvl)
    dim = get_double_dimension(vrblvl)
    print('the dimension :', dim)
    org = "x*y - 1;"
    idx = 1
    set_double_polynomial(idx, dim, org, vrblvl)
    pol = get_double_polynomial(idx, vrblvl)
    print('the retrieved polynomial :', pol)
    smb = string_of_symbols(100, vrblvl)
    print('the list of symbols :', smb)
    return int(len(smb) != 2)

def test_double_double_polynomial(vrblvl=0):
    """
    Tests the setting and getting of a polynomial,
    in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_double_polynomial ...')
    set_double_double_dimension(2, vrblvl)
    dim = get_double_double_dimension(vrblvl)
    print('the dimension :', dim)
    org = "x*y - 1;"
    idx = 1
    set_double_double_polynomial(idx, dim, org, vrblvl)
    pol = get_double_double_polynomial(idx, vrblvl)
    print('the retrieved polynomial :', pol)
    smb = string_of_symbols(100, vrblvl)
    print('the list of symbols :', smb)
    return int(len(smb) != 2)

def test_quad_double_polynomial(vrblvl=0):
    """
    Tests the setting and getting of a polynomial,
    in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_quad_double_polynomial ...')
    set_quad_double_dimension(2, vrblvl)
    dim = get_quad_double_dimension(vrblvl)
    print('the dimension :', dim)
    org = "x*y - 1;"
    idx = 1
    set_quad_double_polynomial(idx, dim, org, vrblvl)
    pol = get_quad_double_polynomial(idx, vrblvl)
    print('the retrieved polynomial :', pol)
    smb = string_of_symbols(100, vrblvl)
    print('the list of symbols :', smb)
    return int(len(smb) != 2)

def test_double_laurent_polynomial(vrblvl=0):
    """
    Tests the setting and getting of a Laurent polynomial,
    in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_laurent_polynomial ...')
    set_double_laurent_dimension(2, vrblvl)
    dim = get_double_laurent_dimension(vrblvl)
    print('the dimension :', dim)
    org = "x*y^(-3) - 1;"
    idx = 1
    set_double_laurent_polynomial(idx, dim, org, vrblvl)
    pol = get_double_laurent_polynomial(idx, vrblvl)
    print('the retrieved polynomial :', pol)
    smb = string_of_symbols(100, vrblvl)
    print('the list of symbols :', smb)
    return int(len(smb) != 2)

def test_double_double_laurent_polynomial(vrblvl=0):
    """
    Tests the setting and getting of a Laurent polynomial,
    in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_double_laurent_polynomial ...')
    set_double_double_laurent_dimension(2, vrblvl)
    dim = get_double_double_laurent_dimension(vrblvl)
    print('the dimension :', dim)
    org = "x*y^(-3) - 1;"
    idx = 1
    set_double_double_laurent_polynomial(idx, dim, org, vrblvl)
    pol = get_double_double_laurent_polynomial(idx, vrblvl)
    print('the retrieved polynomial :', pol)
    smb = string_of_symbols(100, vrblvl)
    print('the list of symbols :', smb)
    return int(len(smb) != 2)

def test_quad_double_laurent_polynomial(vrblvl=0):
    """
    Tests the setting and getting of a Laurent polynomial,
    in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_quad_double_laurent_polynomial ...')
    set_quad_double_laurent_dimension(2, vrblvl)
    dim = get_quad_double_laurent_dimension(vrblvl)
    print('the dimension :', dim)
    org = "x*y^(-3) - 1;"
    idx = 1
    set_quad_double_laurent_polynomial(idx, dim, org, vrblvl)
    pol = get_quad_double_laurent_polynomial(idx, vrblvl)
    print('the retrieved polynomial :', pol)
    smb = string_of_symbols(100, vrblvl)
    print('the list of symbols :', smb)
    return int(len(smb) != 2)

def test_double_system(vrblvl=0):
    """
    Tests the setting and getting of a system,
    in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_system ...')
    polynomials = ["x^3 + 2*x*y - 1;", "x + y - 1;", "x - 1;"]
    dim = number_of_symbols(polynomials, vrblvl)
    print('number of symbols :', dim)
    if dim == len(polynomials):
        print('The system is square.')
    else:
        print('number of polynomials :', len(polynomials))
        print('  number of variables :', dim)
        print('The system is not square.')
    set_double_system(dim, polynomials, vrblvl)
    pols = get_double_system(vrblvl)
    print('the retrieved polynomials :')
    for pol in pols:
        print(pol)
    return int(len(pols) != 3)

def test_double_double_system(vrblvl=0):
    """
    Tests the setting and getting of a system,
    in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_double_system ...')
    polynomials = ["x^3 + 2*x*y - 1;", "x + y - 1/3;", "x - 1;"]
    dim = number_of_symbols(polynomials, vrblvl)
    print('number of symbols :', dim)
    if dim == len(polynomials):
        print('The system is square.')
    else:
        print('number of polynomials :', len(polynomials))
        print('  number of variables :', dim)
        print('The system is not square.')
    set_double_double_system(dim, polynomials, vrblvl)
    pols = get_double_double_system(vrblvl)
    print('the retrieved polynomials :')
    for pol in pols:
        print(pol)
    return int(len(pols) != 3)

def test_quad_double_system(vrblvl=0):
    """
    Tests the setting and getting of a system,
    in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_quad_double_system ...')
    polynomials = ["x^3 + 2*x*y - 1;", "x + y - 1/3;", "y - 1;"]
    dim = number_of_symbols(polynomials, vrblvl)
    print('number of symbols :', dim)
    if dim == len(polynomials):
        print('The system is square.')
    else:
        print('number of polynomials :', len(polynomials))
        print('  number of variables :', dim)
        print('The system is not square.')
    set_quad_double_system(dim, polynomials, vrblvl)
    pols = get_quad_double_system(vrblvl)
    print('the retrieved polynomials :')
    for pol in pols:
        print(pol)
    return int(len(pols) != 3)

def test_double_laurent_system(vrblvl=0):
    """
    Tests the setting and getting of a laurent system,
    in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_laurent_system ...')
    polynomials = ["x^(-3) + 2*x*y - 1;", "x + y^(-1) - 1;", "x - 1;"]
    dim = number_of_symbols(polynomials, vrblvl)
    print('number of symbols :', dim)
    if dim == len(polynomials):
        print('The system is square.')
    else:
        print('number of polynomials :', len(polynomials))
        print('  number of variables :', dim)
        print('The system is not square.')
    set_double_laurent_system(dim, polynomials, vrblvl)
    pols = get_double_laurent_system(vrblvl)
    print('the retrieved polynomials :')
    for pol in pols:
        print(pol)
    return int(len(pols) != 3)

def test_double_double_laurent_system(vrblvl=0):
    """
    Tests the setting and getting of a laurent system,
    in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('test_double_double_laurent_system ...')
    polynomials = ["x^(-3) + 2*x*y - 1;", "x + y**(-1) - 1/3;", "x - 1;"]
    dim = number_of_symbols(polynomials, vrblvl)
    print('number of symbols :', dim)
    if dim == len(polynomials):
        print('The system is square.')
    else:
        print('number of polynomials :', len(polynomials))
        print('  number of variables :', dim)
        print('The system is not square.')
    set_double_double_laurent_system(dim, polynomials, vrblvl)
    pols = get_double_double_laurent_system(vrblvl)
    print('the retrieved polynomials :')
    for pol in pols:
        print(pol)
    return len(pols) != 3

def test_quad_double_laurent_system(vrblvl=0):
    """
    Tests the setting and getting of a laurent system,
    in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_quad_double_laurent_system ...')
    polynomials = ["x^(-3) + 2*x*y - 1;", "x + y**(-1) - 1/3;", "y - 1;"]
    dim = number_of_symbols(polynomials, vrblvl)
    print('number of symbols :', dim)
    if dim == len(polynomials):
        print('The system is square.')
    else:
        print('number of polynomials :', len(polynomials))
        print('  number of variables :', dim)
        print('The system is not square.')
    set_quad_double_laurent_system(dim, polynomials, vrblvl)
    pols = get_quad_double_laurent_system(vrblvl)
    print('the retrieved polynomials :')
    for pol in pols:
        print(pol)
    return int(len(pols) != 3)

def test_double_syspool(vrblvl=0):
    """
    Tests the systems pool in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_syspool ...')
    initialize_double_syspool(3, vrblvl)
    dim = size_double_syspool(vrblvl)
    print('The size of the systems pool :', dim)
    pol1 = ['t - 1;']
    set_double_system(1, pol1, vrblvl)
    copy_to_double_syspool(1)
    pol2 = ['t - 2;']
    set_double_system(1, pol2, vrblvl)
    copy_to_double_syspool(2)
    pol3 = ['t - 3;']
    set_double_system(1, pol3, vrblvl)
    copy_to_double_syspool(3)
    for i in range(1, dim+1):
        clear_double_system(vrblvl)
        copy_from_double_syspool(i)
        pols = get_double_system(vrblvl)
        print('system at', i, 'in the pool :', pols)
    clear_double_syspool(vrblvl)
    return int(dim != 3)

def test_double_double_syspool(vrblvl=0):
    """
    Tests the systems pool in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_double_syspool ...')
    initialize_double_double_syspool(3, vrblvl)
    dim = size_double_double_syspool(vrblvl)
    print('The size of the systems pool :', dim)
    pol1 = ['t - 1/3;']
    set_double_double_system(1, pol1, vrblvl)
    copy_to_double_double_syspool(1)
    pol2 = ['t - 2/3;']
    set_double_double_system(1, pol2, vrblvl)
    copy_to_double_double_syspool(2)
    pol3 = ['t - 1;']
    set_double_double_system(1, pol3, vrblvl)
    copy_to_double_double_syspool(3)
    for i in range(1, dim+1):
        clear_double_double_system(vrblvl)
        copy_from_double_double_syspool(i)
        pols = get_double_double_system(vrblvl)
        print('system at', i, 'in the pool :', pols)
    clear_double_double_syspool(vrblvl)
    return int(dim != 3)

def test_quad_double_syspool(vrblvl=0):
    """
    Tests the systems pool in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_quad_double_syspool ...')
    initialize_quad_double_syspool(3, vrblvl)
    dim = size_quad_double_syspool(vrblvl)
    print('The size of the systems pool :', dim)
    pol1 = ['t - 1/3;']
    set_quad_double_system(1, pol1, vrblvl)
    copy_to_quad_double_syspool(1)
    pol2 = ['t - 2/3;']
    set_quad_double_system(1, pol2, vrblvl)
    copy_to_quad_double_syspool(2)
    pol3 = ['t - 1;']
    set_quad_double_system(1, pol3, vrblvl)
    copy_to_quad_double_syspool(3)
    for i in range(1, dim+1):
        clear_quad_double_system(vrblvl)
        copy_from_quad_double_syspool(i)
        pols = get_quad_double_system(vrblvl)
        print('system at', i, 'in the pool :', pols)
    clear_quad_double_syspool(vrblvl)
    return int(dim != 3)

def test_degree_of_double_polynomial(vrblvl=0):
    """
    Tests the degree of a polynomial in double precision.
    """
    if vrblvl > 0:
        print('in test_degree_of_double_polynomial ...')
    pols = ['x^2*y + y^2 + 1;', 'x^2 + y^2 - 1;']
    if vrblvl > 0:
        print('in test_degree_of_double_polynomial ...')
    set_double_system(len(pols), pols, vrblvl)
    degsum = 0
    for (idx, pol) in enumerate(pols):
        print('index :', idx)
        deg = degree_of_double_polynomial(idx+1, vrblvl)
        print('degree of', pol, ':', deg)
        degsum = degsum + deg
    return int(degsum != 5)

def test_double_number_terms(vrblvl=0):
    """
    Tests the number of terms in a polynomial system
    set in double precision.
    """
    if vrblvl > 0:
        print('in test_double_number_terms ...')
    pols = ['x^2*y;', 'x^2 + y^2;', 'x + z + y;']
    set_double_system(len(pols), pols, vrblvl)
    fail = 0
    for idx in range(len(pols)):
        nbterms = get_double_number_terms(idx+1, vrblvl)
        if vrblvl > 0:
            print('number of terms in equation', idx+1, ':', nbterms)
        fail = fail + int(nbterms != idx+1)
    return fail

def test_double_number_laurent_terms(vrblvl=0):
    """
    Tests the number of terms in a Laurent system
    set in double precision.
    """
    if vrblvl > 0:
        print('in test_double_number_laurent_terms ...')
    pols = ['x^2*y;', 'x^2 + y^2;', 'x + z + y;']
    set_double_laurent_system(len(pols), pols, vrblvl)
    fail = 0
    for idx in range(len(pols)):
        nbterms = get_double_number_laurent_terms(idx+1, vrblvl)
        if vrblvl > 0:
            print('number of terms in equation', idx+1, ':', nbterms)
        fail = fail + int(nbterms != idx+1)
    return fail

def test_double_double_number_terms(vrblvl=0):
    """
    Tests the number of terms in a polynomial system
    set in double double precision.
    """
    if vrblvl > 0:
        print('in test_double_double_number_terms ...')
    pols = ['x^2*y;', 'x^2 + y^2;', 'x + z + y;']
    set_double_double_system(len(pols), pols, vrblvl)
    fail = 0
    for idx in range(len(pols)):
        nbterms = get_double_double_number_terms(idx+1, vrblvl)
        if vrblvl > 0:
            print('number of terms in equation', idx+1, ':', nbterms)
        fail = fail + int(nbterms != idx+1)
    return fail

def test_double_double_number_laurent_terms(vrblvl=0):
    """
    Tests the number of terms in a Laurent system
    set in double double precision.
    """
    if vrblvl > 0:
        print('in test_double_double_number_laurent_terms ...')
    pols = ['x^2*y;', 'x^2 + y^2;', 'x + z + y;']
    set_double_double_laurent_system(len(pols), pols, vrblvl)
    fail = 0
    for idx in range(len(pols)):
        nbterms = get_double_double_number_laurent_terms(idx+1, vrblvl)
        if vrblvl > 0:
            print('number of terms in equation', idx+1, ':', nbterms)
        fail = fail + int(nbterms != idx+1)
    return fail

def test_quad_double_number_terms(vrblvl=0):
    """
    Tests the number of terms in a polynomial system
    set in quad double precision.
    """
    if vrblvl > 0:
        print('in test_quad_double_number_terms ...')
    pols = ['x^2*y;', 'x^2 + y^2;', 'x + z + y;']
    set_quad_double_system(len(pols), pols, vrblvl)
    fail = 0
    for idx in range(len(pols)):
        nbterms = get_quad_double_number_terms(idx+1, vrblvl)
        if vrblvl > 0:
            print('number of terms in equation', idx+1, ':', nbterms)
        fail = fail + int(nbterms != idx+1)
    return fail

def test_quad_double_number_laurent_terms(vrblvl=0):
    """
    Tests the number of terms in a Laurent system
    set in quad double precision.
    """
    if vrblvl > 0:
        print('in test_quad_double_number_laurent_terms ...')
    pols = ['x^2*y;', 'x^2 + y^2;', 'x + z + y;']
    set_quad_double_laurent_system(len(pols), pols, vrblvl)
    fail = 0
    for idx in range(len(pols)):
        nbterms = get_quad_double_number_laurent_terms(idx+1, vrblvl)
        if vrblvl > 0:
            print('number of terms in equation', idx+1, ':', nbterms)
        fail = fail + int(nbterms != idx+1)
    return fail

def test_is_square(vrblvl=0):
    """
    Tests on catching an omitted comma between the polynomials.
    """
    pols = ['x^2 + 4*y^2 - 4;' '2*y^2 - x;']
    retval = check_semicolons(pols, vrblvl)
    fail = int(retval != 1)
    retval = is_square(pols, vrblvl)
    if retval:
        print('The system is square.')
    else:
        print('The system is not square.')
    fail = fail + int(retval)
    return fail

def main():
    """
    Runs tests on the set/get polynomials.
    """
    lvl = 1
    fail = test_double_polynomial(lvl)
    fail = fail + test_double_double_polynomial(lvl)
    fail = fail + test_quad_double_polynomial(lvl)
    fail = fail + test_double_laurent_polynomial(lvl)
    fail = fail + test_double_double_laurent_polynomial(lvl)
    fail = fail + test_quad_double_laurent_polynomial(lvl)
    fail = fail + test_double_system(lvl)
    fail = fail + test_double_double_system(lvl)
    fail = fail + test_quad_double_system(lvl)
    fail = fail + test_double_laurent_system(lvl)
    fail = fail + test_double_double_laurent_system(lvl)
    fail = fail + test_quad_double_laurent_system(lvl)
    fail = fail + test_double_syspool(lvl)
    fail = fail + test_double_double_syspool(lvl)
    fail = fail + test_quad_double_syspool(lvl)
    fail = fail + test_degree_of_double_polynomial(lvl)
    fail = fail + test_double_number_terms(lvl)
    fail = fail + test_double_number_laurent_terms(lvl)
    fail = fail + test_double_double_number_terms(lvl)
    fail = fail + test_double_double_number_laurent_terms(lvl)
    fail = fail + test_quad_double_number_terms(lvl)
    fail = fail + test_quad_double_number_laurent_terms(lvl)
    fail = fail + test_is_square(lvl)
    if fail == 0:
        print('=> All tests passed.')
    else:
        print('Number of failed tests :', fail)

if __name__=="__main__":
    main()



"""
Numerical Schubert calculus defines homotopies for enumerative geometry.
"""
from ctypes import c_int32, c_double, pointer, create_string_buffer
from random import uniform
from math import pi, sin, cos
from phcpy.version import get_phcfun
from phcpy.version import int4a2nbr, nbr2int4a, int4a2str
from phcpy.polynomials import get_double_polynomial
from phcpy.polynomials import get_double_system
from phcpy.solutions import get_double_solutions, clear_double_solutions
from phcpy.solutions import verify

def pieri_root_count(mdim, pdim, qdeg=0, vrblvl=0):
    r"""
    Computes the number of *pdim*-plane producing maps of degree *qdeg*
    that meet *mdim*-planes at mdim*pdim + qdeg*(mdim+pdim) points.
    """
    if vrblvl > 0:
        print('in pieri_root_count, m :', mdim, end='')
        print(', p :', pdim, end='')
        print(', q :', qdeg)
    phc = get_phcfun(vrblvl-1)
    apars = int4a2nbr([mdim, pdim, qdeg], vrblvl=vrblvl-1)
    roco = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> pieri_root_count calls phc ...')
        print('apars =', nbr2int4a(apars))
    retval = phc(223, apars, roco, ccc, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
        print('the Pieri root count :', roco[0])
    return roco[0]

def pieri_localization_poset(mdim, pdim, qdeg=0, size=10240, vrblvl=0):
    r"""
    Returns the string representation of the localization poset
    used to compute the Pieri root count, for the number of *pdim*-plane
    producing maps of degree *qdeg* that meet *mdim*-planes at
    mdim*pdim + qdeg*(mdim+pdim) points.
    The size of the string representation is given on input in size.
    """
    if vrblvl > 0:
        print('in pieri_root_count, m :', mdim, end='')
        print(', p :', pdim, end='')
        print(', q :', qdeg)
    phc = get_phcfun(vrblvl-1)
    apars = int4a2nbr([mdim, pdim, qdeg], vrblvl=vrblvl-1)
    poset = create_string_buffer(b"", 4*size)
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> pieri_localization_poset calls phc ...')
        print('apars =', nbr2int4a(apars))
    retval = phc(224, apars, poset, ccc, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
    allresult = int4a2str(poset, (vrblvl > 0))
    endidx = allresult.find('\0')
    result = allresult[:endidx]
    if vrblvl > 0:
        print('the localization poset :')
        print(result)
    return result

def resolve_schubert_conditions(ndim, kdim, brackets, vrblvl=0):
    r"""
    In n-dimensional space we consider k-dimensional planes,
    subject to intersection conditions represented by brackets.
    The *brackets* is a list of brackets.  A bracket is a list
    of as many natural numbers (in the range 1..*ndim*) as *kdim*.
    On return is the formal root count, which is sharp for general flags.
    and the coordinates of the flags, stored row wise in a list
    of real and imaginary parts.
    """
    if vrblvl > 0:
        print('in resolve_schubert_conditions, n :', ndim, end='')
        print(', k :', kdim, end='')
        print(', #brackets :', len(brackets))
    size = kdim*len(brackets)
    cds = (c_int32 * size)()
    if vrblvl > 0:
        print('conditions :', cds[:])
    idx = 0
    for bracket in brackets:
        for nbr in bracket:
            cds[idx] = c_int32(nbr)
            idx = idx + 1
    if vrblvl > 0:
        print('conditions :', cds[:])
    phc = get_phcfun(vrblvl-1)
    apars = int4a2nbr([ndim, kdim, len(brackets)], vrblvl=vrblvl-1)
    brk = pointer(cds)
    roco = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> resolve_schubert_conditions calls phc ...')
        print('apars =', nbr2int4a(apars))
    retval = phc(228, apars, brk, roco, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
        print('the root count :', int(roco[0]))
    return int(roco[0])

def real_osculating_planes(mdim, pdim, qdeg=0, vrblvl=0):
    """
    Returns m*p + qdeg*(m+p) real m-planes osculating
    a rational normal curve.
    The value of the verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in real_osculating planes, m :', mdim, end='')
        print(', p :', pdim, end='')
        print(', q :', qdeg)
    dim = mdim*pdim + qdeg*(mdim+pdim)
    pts = [uniform(-1, +1) for _ in range(dim)]
    if vrblvl > 0:
        print('the points :', pts)
    size = dim*mdim*(mdim+pdim)
    cpts = (c_double*size)()
    for (idx, point) in enumerate(pts):
        cpts[idx] = c_double(point)
    phc = get_phcfun(vrblvl-1)
    apars = int4a2nbr([mdim, pdim, qdeg], vrblvl=vrblvl-1)
    bbb = pointer(c_int32(0))
    cff = pointer(cpts)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> real_osculating_planes calls phc ...')
        print('apars =', nbr2int4a(apars))
    retval = phc(226, apars, bbb, cff, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
    idx = 0
    planes = []
    for k in range(0, dim):
        plane = []
        for _ in range(0, mdim+pdim):
            row = []
            for _ in range(0, mdim):
                row.append(cff[0][idx])
                idx = idx + 1
            plane.append(row)
        planes.append(plane)
        if vrblvl > 0:
            print('the coefficients of the plane', k, ':')
            for row in plane:
                print(row)
    return planes

def random_complex_matrix(nbrows, nbcols):
    r"""
    Returns a random *nbrows*-by-*nbcols* matrix
    with randomly generated complex coefficients
    on the unit circle, as a list of *rows*.
    """
    result = []
    for _ in range(0, nbrows):
        angles = [uniform(0, 2*pi) for _ in range(nbcols)]
        cols = [complex(cos(a), sin(a)) for a in angles]
        result.append(cols)
    return result

def random_complex_matrices(nbr, nbrows, nbcols):
    r"""
    Returns a list of matrix of length *nbr*,
    all of dimension *nbrows* by *nbcols*.
    """
    result = []
    for _ in range(nbr):
        result.append(random_complex_matrix(nbrows, nbcols))
    return result

def make_pieri_system(mdim, pdim, qdeg, planes, is_real=False, vrblvl=0):
    """
    Makes the polynomial system defined by the mdim-planes
    in the list planes.
    """
    if vrblvl > 0:
        print('in make_pieri_system, m :', mdim, end='')
        print(', p :', pdim, end='')
        print(', q :', qdeg)
    phc = get_phcfun(vrblvl-1)
    apars = int4a2nbr([mdim, pdim, qdeg], vrblvl=vrblvl-1)
    bbb = pointer(c_int32(0))
    dim = mdim*pdim + qdeg*(mdim+pdim)
    size = 2*dim*mdim*(mdim+pdim)
    cpts = (c_double*size)()
    idx = 0
    for plane in planes:
        for row in plane:
            for nbr in row:
                if is_real:
                    cpts[idx] = c_double(nbr)
                    idx = idx + 2
                else:
                    cpts[idx] = c_double(nbr.real)
                    idx = idx + 1
                    cpts[idx] = c_double(nbr.imag)
                    idx = idx + 1
    cff = pointer(cpts)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> make_pieri_system calls phc ...')
        print('apars =', nbr2int4a(apars))
    retval = phc(227, apars, bbb, cff, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
    result = []
    if qdeg == 0:
        for i in range(1, mdim*pdim+1):
            result.append(get_double_polynomial(i, vrblvl))
    return result

def run_pieri_homotopies(mdim, pdim, qdeg, planes, vrblvl=0, *pts):
    r"""
    Computes the number of *pdim*-plane producing maps of degree *qdeg*
    that meet *mdim*-planes at mdim*pdim + qdeq*(mdim+pdim) points.
    For *qdeg* = 0, there are no interpolation points in *pts*.
    The planes and pts are assumed to be complex.
    """
    if vrblvl > 0:
        print('in run_pieri_homotopies, m :', mdim, end='')
        print(', p :', pdim, end='')
        print(', q :', qdeg)
    roco = pieri_root_count(mdim, pdim, qdeg, vrblvl)
    if vrblvl > 0:
        print('Pieri root count for', (mdim, pdim, qdeg), 'is', roco)
    clear_double_solutions(vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    apars = int4a2nbr([mdim, pdim, qdeg], vrblvl=vrblvl-1)
    broco = pointer(c_int32(0))
    dim = mdim*pdim + qdeg*(mdim+pdim)
    if qdeg == 0:
        size = 2*dim*mdim*(mdim+pdim)
    else:
        size = 2*len(pts[0]) + 2*dim*mdim*(mdim+pdim)
    cpts = (c_double*size)()
    idx = 0
    if qdeg > 0:            # points are in rows of a matrix
        for row in pts[0]:
            for cffpts in row:
                cpts[idx] = c_double(cffpts.real)
                idx = idx + 1
                cpts[idx] = c_double(cffpts.imag)
                idx = idx + 1
    for plane in planes:
        for row in plane:
            for nbr in row:
                cpts[idx] = c_double(nbr.real)
                idx = idx + 1
                cpts[idx] = c_double(nbr.imag)
                idx = idx + 1
    cff = pointer(cpts)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> make_pieri_system calls phc ...')
        print('apars =', nbr2int4a(apars))
    retval = phc(225, apars, broco, cff, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
        print('the root count :', broco[0])
    if vrblvl > 0:
        print('making the system ...')
    pols = []
    if qdeg == 0:
        for i in range(1, mdim*pdim+1):
            pols.append(get_double_polynomial(i, vrblvl-1))
    else:
        for i in range(1, mdim*pdim+qdeg*(mdim+pdim)+1):
            pols.append(get_double_polynomial(i, vrblvl-1))
    if vrblvl > 0:
        print('the system :')
        for pol in pols:
            print(pol)
        print('root count :', roco)
    sols = get_double_solutions(vrblvl-1)
    if vrblvl > 0:
        print('the solutions :')
        for sol in sols:
            print(sol)
    return (pols, sols)

def double_littlewood_richardson_homotopies(ndim, kdim, brackets, \
    verbose=True, vrfcnd=False, minrep=True, tosqr=False, outputfilename='',
    vrblvl=0):
    r"""
    In n-dimensional space we consider k-dimensional planes,
    subject to intersection conditions represented by brackets.
    The parameters *ndim* and *kdim* give values for n and k respectively.
    The parameter brackets is a list of brackets.  A bracket is a list
    of as many natural numbers (in the range 1..*ndim*) as *kdim*.
    The Littlewood-Richardson homotopies compute k-planes that
    meet the flags at spaces of dimensions prescribed by the brackets,
    in standard double precision.  Four options are passed as Booleans:

    *verbose*: for adding extra output during computations,

    *vrfcnd*: for extra diagnostic verification of Schubert conditions,

    *minrep*: for a minimial representation of the problem formulation,

    *tosqr*: to square the overdetermined systems.

    On return is a 4-tuple.  The first item of the tuple is the
    formal root count, sharp for general flags, then as second
    item the coordinates of the flags.  The coordinates of the
    flags are stored row wise in a list of real and imaginary parts.
    The third and fourth item of the tuple on return are respectively
    the polynomial system that has been solved and its solutions.
    The length of the list of solution should match the root count.
    """
    if vrblvl > 0:
        print('in double_littlewood_richardson_homotopies ...')
        print('ndim :', ndim, '  kdim :', kdim)
        print('the brackets :')
        for bracket in brackets:
            print(bracket)
    clear_double_solutions(vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    adims = (c_int32 * 8)()
    adims[0] = c_int32(ndim)
    adims[1] = c_int32(kdim)
    adims[2] = c_int32(len(brackets))
    adims[3] = c_int32(int(verbose))
    adims[4] = c_int32(int(vrfcnd))
    adims[5] = c_int32(len(outputfilename))
    adims[6] = c_int32(int(minrep))
    adims[7] = c_int32(int(tosqr))
    pdims = pointer(adims)
    nbc = len(brackets)*kdim
    bcnd = (c_int32 * nbc)()
    idx = 0
    for bracket in brackets:
        for num in bracket:
            bcnd[idx] = c_int32(num)
            idx = idx + 1
    pcnd = pointer(bcnd)
    size = 4*(len(brackets) - 2)*ndim*ndim+1
    name = (c_double * size)()
    for (idx, letter) in enumerate(outputfilename):
        name[idx] = letter
    pname = pointer(name)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_littlewood_richardson_homotopies calls phc ...')
    retval = phc(229, pdims, pcnd, pname, vrb)
    if vrblvl > 0:
        print('the return value of phc :', retval)
    roco = int(pname[0][0])
    if vrblvl > 0:
        print('the root count :', roco)
    allcffs = pname[0][1:size]
    flgs = []
    for idx in range(size, 2):
        realnbr = allcffs[0][idx]
        imagnbr = allcffs[0][idx+1]
        flgs.append(complex(realnbr, imagnbr))
    fsys = get_double_system(vrblvl-1)
    sols = get_double_solutions(vrblvl-1)
    return (roco, flgs, fsys, sols)

def test_pieri_count(vrblvl=0):
    """
    Tests the Pieri root count.
    """
    if vrblvl > 0:
        print('in test_pieri ...')
    roco = pieri_root_count(2, 2, 1, vrblvl)
    poset = pieri_localization_poset(2, 2, 1, vrblvl=vrblvl)
    if vrblvl > 0:
        print('The pieri localization poset :')
        print(poset)
    if roco == 8:
        if vrblvl > 0:
            print('Pieri root count', roco, 'is correct.')
        return 0
    if vrblvl > 0:
        print('Pieri root count', roco, 'is wrong.')
    return 1

def test_littlewood_richardson_rule(vrblvl=0):
    """
    Tests the Littlewood-Richardson rule.
    """
    if vrblvl > 0:
        print('in test_littlewood_richardson_rule ...')
    brk = [[2, 4, 6], [2, 4, 6], [2, 4, 6]]
    roco = resolve_schubert_conditions(6, 3, brk, vrblvl)
    if roco == 2:
        if vrblvl > 0:
            print('Littlewood-Richardson root count :', roco, 'is correct.')
        return 0
    if vrblvl > 0:
        print('Littlewood-Richardson root count :', roco, 'is wrong.')
    return 1

def test_pieri_problem(vrblvl=0):
    """
    Tests the making of a Pieri problem.
    """
    if vrblvl > 0:
        print('in test_pieri_problem ...')
    planes = real_osculating_planes(2, 2, 0, vrblvl)
    pols = make_pieri_system(2, 2, 0, planes, is_real=True, vrblvl=vrblvl)
    if vrblvl > 0:
        for pol in pols:
            print(pol)
    if vrblvl > 0:
        print('making a complex problem ...')
    planes = random_complex_matrices(2*2, 4, 2)
    if vrblvl > 0:
        for (idx, plane) in enumerate(planes):
            print('plane', idx, ':')
            for row in plane:
                print(row)
    pols = make_pieri_system(2, 2, 0, planes, is_real=False, vrblvl=vrblvl)
    if vrblvl > 0:
        for pol in pols:
            print(pol)
    return 0

def test_pieri_homotopies(vrblvl=0):
    """
    Tests running the Pieri homotopies on the four lines problem.
    """
    if vrblvl > 0:
        print('in test_pieri_homotopies ...')
    planes = random_complex_matrices(2*2, 4, 2)
    if vrblvl > 0:
        for (idx, plane) in enumerate(planes):
            print('plane', idx, ':')
            for row in plane:
                print(row)
    pols, sols = run_pieri_homotopies(2, 2, 0, planes, vrblvl=vrblvl)
    if vrblvl > 0:
        for pol in pols:
            print(pol)
        for sol in sols:
            print(sol)
    err = verify(pols, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error sum :', err)
    if len(sols) == 2 and err < 1.0e-10:
        if vrblvl > 0:
            print('Found 2 solutions and error is okay.')
        return 0
    if len(sols) != 2:
        if vrblvl > 0:
            print('Number of solutions is not 2 :', len(sols))
        return 1
    if err >= 1.0e-10:
        if vrblvl > 0:
            print('The error is too large.')
    return 1

def test_littlewood_richardson_homotopies(vrblvl=0):
    """
    Performs a test on the Littlewood-Richardson homotopies.
    """
    if vrblvl > 0:
        print('in test_littlewood_richardson_homotopies ...')
    brk = [[2, 4, 6], [2, 4, 6], [2, 4, 6]]
    (roco, flags, fsys, sols) \
        = double_littlewood_richardson_homotopies\
             (6, 3, brk, vrblvl=vrblvl)
    if vrblvl > 0:
        print('the root count :', roco)
    if vrblvl > 0:
        print('the flags :', flags)
    if vrblvl > 0:
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(fsys, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error sum :', err)
    if len(sols) == 2 and err < 1.0e-10:
        if vrblvl > 0:
            print('Found 2 solutions and error is okay.')
        return 0
    if len(sols) != 2:
        if vrblvl > 0:
            print('Number of solutions is not 2 :', len(sols))
        return 1
    if err >= 1.0e-10:
        if vrblvl > 0:
            print('The error is too large.')
    return 1

def test_pieri_curves(vrblvl=0):
    """
    Computes all line producing curves that meet given lines in 3-space,
    at given interpolation points, computed with Pieri homotopies.
    """
    if vrblvl > 0:
        print('in test_pieri_curves ...')
    (mdim, pdim, qdeg) = (2, 2, 1)
    dim = mdim*pdim + qdeg*(mdim+pdim)
    roco = pieri_root_count(mdim, pdim, qdeg)
    if vrblvl > 0:
        print('the root count :', roco)
    pls = [random_complex_matrix(mdim+pdim, mdim) for _ in range(dim)]
    points = random_complex_matrix(dim, 1)
    (fsys, fsols) = run_pieri_homotopies(mdim, pdim, qdeg, pls, vrblvl, points)
    if vrblvl > 0:
        print('number of solutions :', len(fsols))
    fail = int(roco != len(fsols))
    err = verify(fsys, fsols, vrblvl-1)
    if vrblvl > 0:
        print('the error :', err)
    fail = fail + int(err > 1.0e-8)
    return fail

def main():
    """
    Runs some tests.
    """
    lvl = 1
    fail = test_pieri_count(lvl)
    fail = fail + test_littlewood_richardson_rule(lvl)
    fail = fail + test_pieri_problem(lvl)
    fail = fail + test_pieri_homotopies(lvl)
    fail = fail + test_littlewood_richardson_homotopies(lvl)
    fail = fail + test_pieri_curves(lvl)
    if fail == 0:
        print('=> All tests passed.')
    else:
        print('Number of failed tests :', fail)

if __name__=="__main__":
    main()



"""
Exports the blackbox solver for isolated solutions of square systems,
in double, double double, and quad double precision.
Also Laurent systems are accepted on input.
"""
from random import randint, uniform, seed
from math import cos, sin, pi
from ctypes import c_int32, c_double, pointer, create_string_buffer
from phcpy.version import get_phcfun, int4a2nbr, int4a2str
from phcpy.dimension import set_seed
from phcpy.polynomials import set_double_system, set_double_laurent_system
from phcpy.polynomials import set_double_double_system
from phcpy.polynomials import set_double_double_laurent_system
from phcpy.polynomials import set_quad_double_system
from phcpy.polynomials import set_quad_double_laurent_system
from phcpy.polynomials import check_semicolons
from phcpy.polynomials import is_square, number_of_symbols
from phcpy.solutions import clear_double_solutions
from phcpy.solutions import clear_double_double_solutions
from phcpy.solutions import clear_quad_double_solutions
from phcpy.solutions import get_double_solutions
from phcpy.solutions import get_double_double_solutions
from phcpy.solutions import get_quad_double_solutions
from phcpy.solutions import write_double_solutions
from phcpy.solutions import write_double_double_solutions
from phcpy.solutions import write_quad_double_solutions
from phcpy.solutions import formdictlist
from phcpy.volumes import stable_mixed_volume

def random_trinomials(vrblvl=0):
    """
    Returns a system of two trinomials equations for testing.
    A trinomial consists of three monomials in two variables.
    Exponents are uniform between 0 and 5 and coefficients are
    on the complex unit circle.
    """
    if vrblvl > 0:
        print('in random trinomials ...')
    exponents = [(randint(0, 5), randint(0, 5)) for _ in range(0, 6)]
    monomials = [f'x^{xe}*y^{ye}' for (xe, ye) in exponents]
    angles = [uniform(0, 2*pi) for _ in range(0, 6)]
    cff = [f'({cos(a):.14f} {sin(a):+.14f}*i)' for a in angles]
    one = '+'.join(cff[i] + '*' + monomials[i] for i in range(0, 3)) + ';'
    two = '+'.join(cff[i] + '*' + monomials[i] for i in range(3, 6)) + ';'
    return [one, two]

def real_random_trinomials(sys, vrblvl=0):
    r"""
    On input in sys are two random trinonials with complex coefficients,
    in the format what **random_trinomials()** returns.
    On return is a list of two real random trinomials with the same
    monomial structure but with random real coefficients in [-1,+1].
    """
    if vrblvl > 0:
        print('in real_random_trinomials, the system :')
        for pol in sys:
            print(pol)
    result = []
    for pol in sys:
        terms = pol.split(')')
        rpol = ''
        for i in range(1, len(terms)-1):
            rterm = terms[i].split('+')
            cff = f'{uniform(-1, +1):+.17f}'
            rpol = rpol + cff + rterm[0]
        cff = f'{uniform(-1, +1):+.17f}'
        rpol = rpol + cff + terms[len(terms)-1]
        result.append(rpol)
    return result

def solve_double_system(nbtasks=0, mvfocus=0, vrblvl=0):
    """
    Solves the system stored in double precision, where
    nbtasks equals the number of tasks, no multitasking if zero,
    mvfocus equals zero by default and all root counts are computed,
    otherwise, the focus is on mixed volumes and polyhedral homotopies,
    and vrblvl is the verbose level.
    """
    if vrblvl > 0:
        print('in solve_double_system, nbtasks :', nbtasks)
    clear_double_solutions(vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    apars = create_string_buffer(b"", 8)
    broco = create_string_buffer(b"", 8096)
    bpars = int4a2nbr([0, nbtasks, mvfocus], vrblvl=vrblvl-1)
    for i in range(12):
        broco[i] = bpars[i]
    ccc = pointer(c_double(0.0))
    vlvl = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> solve_double_system calls phc', end='')
    retval = phc(77, apars, broco, ccc, vlvl)
    if vrblvl > 0:
        print(', return value :', retval)
    roco = int.from_bytes(apars[0], "big")
    result = int4a2str(broco, vrblvl=vrblvl-1)
    resultend = result.find('\0')
    return (roco, result[:resultend])

def solve_double_double_system(nbtasks=0, mvfocus=0, vrblvl=0):
    """
    Solves the system stored in double double precision, where
    nbtasks equals the number of tasks, no multitasking if zero,
    mvfocus equals zero by default and all root counts are computed,
    otherwise, the focus is on mixed volumes and polyhedral homotopies,
    and vrblvl is the verbose level.
    """
    if vrblvl > 0:
        print("in solve_double_double_system, nbtasks :", nbtasks)
    clear_double_double_solutions(vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    apars = create_string_buffer(b"", 8)
    broco = create_string_buffer(b"", 8096)
    bpars = int4a2nbr([0, nbtasks, mvfocus], vrblvl=vrblvl-1)
    for i in range(12):
        broco[i] = bpars[i]
    ccc = pointer(c_double(0.0))
    vlvl = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> solve_double_double_system calls phc', end='')
    retval = phc(700, apars, broco, ccc, vlvl)
    if vrblvl > 0:
        print(', return value :', retval)
    roco = int.from_bytes(apars[0], "big")
    result = int4a2str(broco, vrblvl=vrblvl-1)
    resultend = result.find('\0')
    return (roco, result[:resultend])

def solve_quad_double_system(nbtasks=0, mvfocus=0, vrblvl=0):
    """
    Solves the system stored in quad double precision, where
    nbtasks equals the number of tasks, no multitasking if zero,
    mvfocus equals zero by default and all root counts are computed,
    otherwise, the focus is on mixed volumes and polyhedral homotopies,
    and vrblvl is the verbose level.
    """
    if vrblvl > 0:
        print("in solve_quad_double_system, nbtasks :", nbtasks)
    clear_quad_double_solutions(vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    apars = create_string_buffer(b"", 8)
    broco = create_string_buffer(b"", 8096)
    bpars = int4a2nbr([0, nbtasks, mvfocus], vrblvl=vrblvl-1)
    for i in range(12):
        broco[i] = bpars[i]
    ccc = pointer(c_double(0.0))
    vlvl = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> solve_quad_double_system calls phc', end='')
    retval = phc(702, apars, broco, ccc, vlvl)
    if vrblvl > 0:
        print(', return value :', retval)
    roco = int.from_bytes(apars[0], "big")
    result = int4a2str(broco, vrblvl=vrblvl-1)
    resultend = result.find('\0')
    return (roco, result[:resultend])

def solve_double_laurent_system(nbtasks=0, mvfocus=0, vrblvl=0):
    """
    Solves the laurent system stored in double precision, where
    nbtasks equals the number of tasks, no multitasking if zero,
    mvfocus equals zero by default and all root counts are computed,
    otherwise, the focus is on mixed volumes and polyhedral homotopies,
    and vrblvl is the verbose level.
    """
    if vrblvl > 0:
        print('in solve_double_laurent_system, nbtasks :', nbtasks)
    clear_double_solutions(vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    apars = create_string_buffer(b"", 8)
    broco = create_string_buffer(b"", 8096)
    bpars = int4a2nbr([0, nbtasks, mvfocus], vrblvl=vrblvl-1)
    for i in range(12):
        broco[i] = bpars[i]
    ccc = pointer(c_double(0.0))
    vlvl = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> solve_double_laurent_system calls phc', end='')
    retval = phc(75, apars, broco, ccc, vlvl)
    if vrblvl > 0:
        print(', return value :', retval)
    roco = int.from_bytes(apars[0], "big")
    result = int4a2str(broco, vrblvl=vrblvl-1)
    resultend = result.find('\0')
    return (roco, result[:resultend])

def solve_double_double_laurent_system(nbtasks=0, mvfocus=0, vrblvl=0):
    """
    Solves the laurent system stored in double double precision, where
    nbtasks equals the number of tasks, no multitasking if zero,
    mvfocus equals zero by default and all root counts are computed,
    otherwise, the focus is on mixed volumes and polyhedral homotopies,
    and vrblvl is the verbose level.
    """
    if vrblvl > 0:
        print('in solve_double_double_laurent_system, nbtasks :', nbtasks)
    clear_double_double_solutions(vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    apars = create_string_buffer(b"", 8)
    broco = create_string_buffer(b"", 8096)
    bpars = int4a2nbr([0, nbtasks, mvfocus], vrblvl=vrblvl-1)
    for i in range(12):
        broco[i] = bpars[i]
    ccc = pointer(c_double(0.0))
    vlvl = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> solve_double_double_laurent_system calls phc', end='')
    retval = phc(701, apars, broco, ccc, vlvl)
    if vrblvl > 0:
        print(', return value :', retval)
    roco = int.from_bytes(apars[0], "big")
    result = int4a2str(broco, vrblvl=vrblvl-1)
    resultend = result.find('\0')
    return (roco, result[:resultend])

def solve_quad_double_laurent_system(nbtasks=0, mvfocus=0, vrblvl=0):
    """
    Solves the laurent system stored in quad double precision, where
    nbtasks equals the number of tasks, no multitasking if zero,
    mvfocus equals zero by default and all root counts are computed,
    otherwise, the focus is on mixed volumes and polyhedral homotopies,
    and vrblvl is the verbose level.
    """
    if vrblvl > 0:
        print('in solve_quad_double_laurent_system, nbtasks :', nbtasks)
    clear_quad_double_solutions(vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    apars = create_string_buffer(b"", 8)
    broco = create_string_buffer(b"", 8096)
    bpars = int4a2nbr([0, nbtasks, mvfocus], vrblvl=vrblvl-1)
    for i in range(12):
        broco[i] = bpars[i]
    ccc = pointer(c_double(0.0))
    vlvl = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> solve_quad_double_laurent_system calls phc', end='')
    retval = phc(703, apars, broco, ccc, vlvl)
    if vrblvl > 0:
        print(', return value :', retval)
    roco = int.from_bytes(apars[0], "big")
    result = int4a2str(broco, vrblvl=vrblvl-1)
    resultend = result.find('\0')
    return (roco, result[:resultend])

def solve_checkin(pols, msg, vrblvl=0):
    r"""
    Checks whether the system defined by the list of strings in *pols*
    is square.  If so, True is returned.  Otherwise, the error message
    in the string *msg* is printed to help the user.
    """
    if vrblvl > 0:
        print('in solve_checkin ...')
    mismatch = check_semicolons(pols, vrblvl-1)
    if mismatch:
        print('Warning: mismatched number of semicolons.')
        return False
    if is_square(pols, vrblvl-1):
        return True
    print(msg)
    dim = number_of_symbols(pols, vrblvl-1)
    neq = len(pols)
    print(f'got {neq} polynomials in {dim} variables.')
    print('Either correct the input, or consider phcpy.decomposition')
    print('to solve polynomial systems that are not square.')
    return False

def solve(pols, tasks=0, mvfocus=0, precision='d',
    checkin=True, dictionary_output=False, vrblvl=0):
    r"""
    Calls the blackbox solver to compute all isolated solutions.
    To compute all solutions, also all positive dimensional solution sets,
    with a numerical irreducible decomposition, see phcpy.decomposition.
    On input in *pols* is a list of strings.
    The number of tasks for multithreading is given by *tasks*.
    The default zero value for *tasks* indicates no multithreading.
    If the *mvfocus* option is set to one, then only mixed volumes
    and polyhedral homotopies will be applied in the solver, and no
    degree bounds will be computed, as is already the case when the
    input system is genuinely laurent and has negative exponents.
    Three levels of precision are supported:

    *d*: standard double precision (1.1e-15 or 2^(-53)),

    *dd*: double double precision (4.9e-32 or 2^(-104)),

    *qd*: quad double precision (1.2e-63 or 2^(-209)).

    If *checkin* (by default), the input *pols* is checked for being square.
    If *dictionary_output*, then on return is a list of dictionaries,
    else the returned list is a list of strings.
    If *vrblvl* is larger than 0, then the names of the procedures
    called in the running of the blackbox solver will be listed.
    """
    if vrblvl > 0:
        print('in solve, tasks :', tasks, end='')
        print(', mvfocus :', mvfocus, end='')
        print(', precision :', precision)
        print('the polynomials on input :')
        for pol in pols:
            print(pol)
    if checkin:
        errmsg = 'The blackbox solver accepts only square systems,'
        if not solve_checkin(pols, errmsg):
            return None
        if tasks < 0:
            print('The number of tasks must be a nonnegative integer.')
            return None
    if precision == 'd':
        set_double_laurent_system(len(pols), pols, vrblvl=vrblvl-1)
        nbr, roco = solve_double_laurent_system(nbtasks=tasks, \
            mvfocus=mvfocus, vrblvl=vrblvl-1)
        if vrblvl > 0:
            print('nbr :', nbr, end='')
            print(', roco :')
            print(roco)
        sols = get_double_solutions(vrblvl=vrblvl-1)
        clear_double_solutions(vrblvl=vrblvl-1)
        if dictionary_output:
            return formdictlist(sols)
        return sols
    if precision == 'dd':
        set_double_double_laurent_system(len(pols), pols, \
            vrblvl=vrblvl-1)
        nbr, roco = solve_double_double_laurent_system(nbtasks=tasks, \
            mvfocus=mvfocus, vrblvl=vrblvl-1)
        if vrblvl > 0:
            print('nbr :', nbr, end='')
            print(', roco :')
            print(roco)
        sols = get_double_double_solutions(vrblvl=vrblvl-1)
        clear_double_double_solutions(vrblvl=vrblvl-1)
        if dictionary_output:
            return formdictlist(sols)
        return sols
    if precision == 'qd':
        set_quad_double_laurent_system(len(pols), pols, \
            vrblvl=vrblvl-1)
        nbr, roco = solve_quad_double_laurent_system(nbtasks=tasks, \
            mvfocus=mvfocus, vrblvl=vrblvl-1)
        if vrblvl > 0:
            print('nbr :', nbr, end='')
            print(', roco :')
            print(roco)
        sols = get_quad_double_solutions(vrblvl=vrblvl-1)
        clear_quad_double_solutions(vrblvl=vrblvl-1)
        if dictionary_output:
            return formdictlist(sols)
        return sols
    print('wrong level of precision, use d, dd, or qd')
    return None

def test_trinomial_solve(vrblvl=0):
    """
    Generates a random trinomial system and solves it.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_trinomial_solve ...')
    seed(12345678)
    pols = random_trinomials(vrblvl)
    if vrblvl > 0:
        print('two random trinomials :')
        print(pols)
    set_seed(12345678)
    (mixvol, stable_mixvol) = stable_mixed_volume(pols, vrblvl-1)
    if vrblvl > 0:
        print('its mixed volume :', mixvol)
        print('its stable mixed volume :', stable_mixvol)
    nbr, roco = solve_double_system(vrblvl=vrblvl)
    if vrblvl > 0:
        print('number of computed solutions :', nbr)
        print('the root counts :')
        print(roco)
    clear_double_solutions(vrblvl-1)
    return int(nbr != stable_mixvol) + int(nbr != 28)

def test_double_solve(vrblvl=0):
    """
    Solves a simple system in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_solve ...')
    polynomials = ["x^3 + 2*x*y - x^2;", "x + y - x^3;"]
    set_double_system(2, polynomials, vrblvl-1)
    nbr, roco = solve_double_system(vrblvl=vrblvl)
    if vrblvl > 0:
        print('number of solutions :', nbr)
        print('root counts :\n', roco)
        write_double_solutions(vrblvl-1)
    sols = get_double_solutions(vrblvl-1)
    if vrblvl > 0:
        print('number of retrieved solutions :', len(sols))
    clear_double_solutions(vrblvl-1)
    return int(len(sols) != 3)

def test_double_double_solve(vrblvl=0):
    """
    Solves a simple system in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_double_solve ...')
    polynomials = ["x^3 + 2*x*y - x^2;", "x + y - x^3;"]
    set_double_double_system(2, polynomials, vrblvl-1)
    nbr, roco = solve_double_double_system(vrblvl=vrblvl)
    if vrblvl > 0:
        print('number of solutions :', nbr)
        print('root counts :\n', roco)
        write_double_double_solutions(vrblvl-1)
    sols = get_double_double_solutions(vrblvl-1)
    if vrblvl > 0:
        print('number of retrieved solutions :', len(sols))
    clear_double_double_solutions(vrblvl-1)
    return int(len(sols) != 3) # is known bug!

def test_quad_double_solve(vrblvl=0):
    """
    Solves a simple system in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_quad_double_solve ...')
    polynomials = ["x^3 + 2*x*y - x^2;", "x + y - x^3;"]
    set_quad_double_system(2, polynomials, vrblvl-1)
    nbr, roco = solve_quad_double_system(vrblvl=vrblvl)
    if vrblvl > 0:
        print("number of solutions :", nbr)
        print("root counts :\n", roco)
        write_quad_double_solutions(vrblvl-1)
    sols = get_quad_double_solutions(vrblvl-1)
    if vrblvl > 0:
        print('number of retrieved solutions :', len(sols))
    clear_quad_double_solutions(vrblvl-1)
    return len(sols) != 3 # is known bug!

def test_double_laurent_solve(vrblvl=0):
    """
    Solves a simple laurent system in double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_laurent_solve ...')
    polynomials = ["x^(-3) + 2*x*y - x;", "x + y^(-2) - x^3;"]
    set_double_laurent_system(2, polynomials, vrblvl-1)
    nbr, roco = solve_double_laurent_system(vrblvl=vrblvl)
    if vrblvl > 0:
        print('number of solutions :', nbr)
        print('root counts :\n', roco)
        write_double_solutions(vrblvl-1)
    sols = get_double_solutions(vrblvl-1)
    if vrblvl > 0:
        print('number of retrieved solutions :', len(sols))
    clear_double_solutions(vrblvl-1)
    return int(len(sols) != 10)

def test_double_double_laurent_solve(vrblvl=0):
    """
    Solves a simple laurent system in double double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_double_laurent_solve ...')
    polynomials = ["x^(-3) + 2*x*y - x;", "x + y^(-2) - x^3;"]
    set_double_double_laurent_system(2, polynomials, vrblvl-1)
    nbr, roco = solve_double_double_laurent_system(vrblvl=vrblvl)
    if vrblvl > 0:
        print('number of solutions :', nbr)
        print('root counts :\n', roco)
        write_double_double_solutions(vrblvl-1)
    sols = get_double_double_solutions(vrblvl-1)
    if vrblvl > 0:
        print('number of retrieved solutions :', len(sols))
    clear_double_double_solutions(vrblvl-1)
    return int(len(sols) != 10)

def test_quad_double_laurent_solve(vrblvl=0):
    """
    Solves a simple laurent system in quad double precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_quad_double_laurent_solve ...')
    polynomials = ["x^(-3) + 2*x*y - x;", "x + y^(-2) - x^3;"]
    set_quad_double_laurent_system(2, polynomials, vrblvl-1)
    nbr, roco = solve_quad_double_laurent_system(vrblvl=vrblvl)
    if vrblvl > 0:
        print('number of solutions :', nbr)
        print('root counts :\n', roco)
        write_quad_double_solutions(vrblvl-1)
    sols = get_quad_double_solutions(vrblvl-1)
    if vrblvl > 0:
        print('number of retrieved solutions :', len(sols))
    clear_quad_double_solutions(vrblvl-1)
    return int(len(sols) != 10)

def test_solve(vrblvl=0):
    """
    Tests the solve function on a simple system.
    """
    if vrblvl > 0:
        print('in test_solve ...')
    polynomials = ["x^3 + 2*x*y - x^2;", "x + y - x^3;"]
    sols = solve(polynomials, vrblvl=vrblvl)
    if vrblvl > 0:
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    fail = int(len(sols) != 3)
    return fail

def main():
    """
    Runs tests on the blackbox solver.
    """
    lvl = 1
    fail = test_trinomial_solve(lvl)
    fail = fail + test_double_solve(lvl)
    fail = fail + test_double_double_solve(lvl)
    fail = fail + test_quad_double_solve(lvl)
    fail = fail + test_double_laurent_solve(lvl)
    fail = fail + test_double_double_laurent_solve(lvl)
    fail = fail + test_quad_double_laurent_solve(lvl)
    fail = fail + test_solve(lvl)
    if fail == 0:
        print('=> All tests passed.')
    else:
        print('Number of failed tests :', fail)

if __name__=='__main__':
    main()
